@@ -1,4 +1,5 @@
 # -*- coding: utf-8 -*-
+import concurrent
 import hashlib
 import logging
 from collections import deque
@@ -10,7 +11,7 @@ from jinja2 import PackageLoader
 from jinja2 import select_autoescape
 
 from .index import Index
-from .s3 import S3
+from .s3_gateway import S3Gateway
 
 
 class BucketDirGenerator:
@@ -28,7 +29,7 @@ class BucketDirGenerator:
             lstrip_blocks=True,
         )
         self.site_name = site_name
-        self.s3_gateway = S3(bucket_name=bucket_name)
+        self.s3_gateway = S3Gateway(bucket_name=bucket_name, logger=self.logger)
 
     @staticmethod
     def generate_ascending_prefixes(directory_key):
@@ -43,7 +44,7 @@ class BucketDirGenerator:
         paths.reverse()
         return paths
 
-    def generate(self, exclude_objects=None, single_threaded=False, target_path=""):
+    def generate(self, extra_exclude_objects=None, single_threaded=False, target_path=""):
         if target_path.startswith("/"):
             target_path = target_path[1:]
 
@@ -51,33 +52,45 @@ class BucketDirGenerator:
             last_slash = target_path.rfind("/")
             target_path = target_path[: last_slash + 1]
 
+        excluded_objects = ["favicon.ico", "index.html"]
+        if extra_exclude_objects:
+            excluded_objects.extend(extra_exclude_objects)
+
         max_workers = 1 if single_threaded else None
 
         with ThreadPoolExecutor(max_workers=max_workers) as executor:
             self.logger.info(
                 f"Generating indexes for {self.s3_gateway.bucket_name} across {executor._max_workers} worker threads."
             )
-            futures = deque([])
-            futures.append(
-                executor.submit(
-                    self.render_and_upload_index_document_to_s3,
-                    target_path,
-                    exclude_objects,
-                    executor,
-                )
-            )
-
-            for prefix in self.generate_ascending_prefixes(target_path):
-                futures.append(
-                    executor.submit(
-                        self.render_and_upload_index_document_to_s3,
-                        prefix,
-                        exclude_objects,
-                    )
-                )
+            folder_dictionary, futures = self.enqueue_folder_discovery(executor, target_path)
 
             self.wait_for_all_futures_recursively(futures)
-            self.logger.info(f"Finished generation.")
+
+            for folder in folder_dictionary.values():
+                futures.append(
+                    executor.submit(self.update_index, folder_dictionary, folder, excluded_objects)
+                )
+            for future in concurrent.futures.as_completed(futures):
+                if future.exception() is not None:
+                    raise future.exception()
+
+        self.logger.info(f"Finished generation.")
+
+    def enqueue_folder_discovery(self, executor, target_path):
+        folder_dictionary = {}
+        futures = deque([])
+        futures.append(
+            executor.submit(
+                self.discover_folder,
+                folder_dictionary,
+                target_path,
+                executor,
+            )
+        )
+        for prefix in self.generate_ascending_prefixes(target_path):
+            futures.append(executor.submit(self.discover_folder, folder_dictionary, prefix))
+
+        return folder_dictionary, futures
 
     def wait_for_all_futures_recursively(self, futures):
         self.logger.debug(f"Waiting for all futures to finish.")
@@ -85,44 +98,63 @@ class BucketDirGenerator:
             sub_futures_array = futures.popleft().result()
             futures.extend(sub_futures_array)
 
-    def render_and_upload_index_document_to_s3(self, prefix, extra_excluded_items, executor=None):
+    def discover_folder(self, folder_dictionary, prefix, executor=None):
         self.logger.debug(f"Rendering index for prefix: '{prefix}'.")
         folder = self.s3_gateway.fetch_folder_content(prefix)
+        folder_dictionary[prefix] = folder
 
         futures = []
         if executor is not None:
             for subdirectory in folder.subdirectories:
                 futures.append(
                     executor.submit(
-                        self.render_and_upload_index_document_to_s3,
+                        self.discover_folder,
+                        folder_dictionary,
                         subdirectory,
-                        extra_excluded_items,
                         executor,
                     )
                 )
 
-        index = Index(prefix, extra_excluded_items)
-        index.items.extend(folder.files)
-        index.folders.extend(folder.subdirectories)
+        return futures
 
-        index_document = index.render(
-            site_name=self.site_name, template_environment=self.template_environment
-        ).encode("utf-8")
+    def update_index(self, folder_dictionary, folder, excluded_objects):
+        key = f"{folder.prefix}index.html"
+        old_hash = folder.get_index_hash()
+        if folder.is_empty(excluded_objects):
+            self.logger.debug(f"Skipping empty folder {key}.")
+            if old_hash:
+                self.s3_gateway.delete_object(key)
+            return
 
-        key = f"{prefix}index.html"
+        def is_folder_in_index(prefix):
+            if folder_dictionary.get(prefix):
+                is_empty = folder_dictionary[prefix].is_empty(excluded_objects)
+                return not is_empty
+            else:
+                return True
 
+        non_empty_subdirectories = list(
+            filter(
+                is_folder_in_index,
+                folder.subdirectories,
+            )
+        )
+
+        index_document = (
+            Index(folder.prefix, folder.files, non_empty_subdirectories, excluded_objects)
+            .render(site_name=self.site_name, template_environment=self.template_environment)
+            .encode("utf-8")
+        )
         new_hash = hashlib.md5(  # nosec # skip bandit check as this is not used for encryption
             index_document
         ).hexdigest()
-        old_hash = folder.get_index_hash()
+
         self.logger.debug(f"{key} comparing existing hash: {old_hash} to new hash: {new_hash}.")
         if old_hash == new_hash:
             self.logger.debug(f"Skipping unchanged index for {key}.")
         else:
-            self.logger.info(f"Uploading index for {key}.")
+
             self.s3_gateway.put_object(
                 body=index_document,
                 key=key,
             )
-
-        return futures
